using System;
using System.Threading.Tasks;
using Newtonsoft.Json;
using System.Net.Http;
using System.Net;
using System.Text;

namespace svr.rpc.server.weather
{
    class CdBridgeServer
    {
        public dBridges.dBridges dbridge;
        public dBridges.remoteprocedure.Crpcserver weatherSvr;
        private string dbApplnKey = 'appkey';
        private string dbAuthURL = 'auth_url';
        private string dbApplnSecret = 'appsecret';
        private string weatherAPPId = 'weather_APPID';

        // Define exposed functions.
        // RPC: the client sends a single request and receives a single response.
        public async void currentWeather(object inparameter, object response)
        {
            dBridges.Utils.dBParams parameter = inparameter as dBridges.Utils.dBParams;
            dBridges.responseHandler.CResponseHandler responsehandler = response as dBridges.responseHandler.CResponseHandler;
            using (var client = new HttpClient()) {
                try {
                    ServicePointManager.ServerCertificateValidationCallback += (sender, cert, chain, sslPolicyErrors) => true;
                    string payload = parameter.inparama;
                    var watch = new System.Diagnostics.Stopwatch();
                    string url = "https://api.openweathermap.org/data/2.5/weather?q=" + payload + "&appid=" + this.weatherAPPId + "&units=metric";
                    var responsedata = await client.GetAsync(new Uri(url));

                    if (!responsedata.IsSuccessStatusCode) {
                        Dictionary<string, object> resultjson = new Dictionary<string, object>();
                        resultjson.Add("statuscode", 1);
                        watch.Stop();
                        resultjson.Add("data", responsedata.ReasonPhrase);
                        resultjson.Add("exectime", watch.ElapsedMilliseconds);
                        jsonstring = JsonConvert.SerializeObject(resultjson);
                        await responsehandler.end(jsonstring);
                    } else {
                        string jsonstring = await responsedata.Content.ReadAsStringAsync();
                        dynamic resp = JsonConvert.DeserializeObject(jsonstring);
                        string responseData = "Weather Condition : " + resp["weather"][0]["main"] + "\n";
                        responseData = responseData + "Temperature : " + resp["main"]["temp"] + "\u00b0C, feels like " + resp["main"]["feels_like"] + "\u00b0C, humidity : " + resp["main"]["humidity"] + "\n";
                        responseData = responseData + "Pressure : " + (resp["main"]["temp"]) + " hPa , Visibility : " + (resp["visibility"]) + " meter, Cloudiness : " + (resp["clouds"]["all"]) + " %" + "\n";
                        responseData = responseData + "Wind : Speed " + (resp["wind"]["speed"]) + " meter/sec , Wind direction : " + (resp["wind"]["deg"]) + " \u00b0" + "\n";
                        responseData = responseData + "Sunrise : " + (resp["sys"]["sunrise"]) + ", Sunset : " + (resp["sys"]["sunset"]) + "\n";
                        Dictionary<string, object> resultjson = new Dictionary<string, object>();
                        resultjson.Add("statuscode", 0);
                        watch.Stop();
                        resultjson.Add("data", responseData);
                        resultjson.Add("exectime", watch.ElapsedMilliseconds);
                        jsonstring = JsonConvert.SerializeObject(resultjson);
                        await responsehandler.end(jsonstring);
                    }
                } catch (Exception e) {
                    Dictionary<string, object> resultjson = new Dictionary<string, object>();
                    resultjson.Add("statuscode", 1);
                    watch.Stop();
                    resultjson.Add("data", "");
                    resultjson.Add("exectime", watch.ElapsedMilliseconds);
                    jsonstring = JsonConvert.SerializeObject(resultjson);
                    await responsehandler.end(jsonstring);
                }
            }
        }

        // Define all exposed functions.
        public void rpcfunction(object eventInfo)
        {
            try {
                Action<object, object> icurrentWeather = this.currentWeather;
                (eventInfo as dBridges.remoteprocedure.Crpcserver).regfn("currentWeather", icurrentWeather);
               
            } catch (Exception e) {
                Console.WriteLine(e);
            }
        }


        public async Task init_dataBridges()
        {
             // Initialize dataBridges client
            this.dbridge = new dBridges.dBridges();

            // Replace your authentication url below, which you have received from dataBridges management portal.
            this.dbridge.auth_url = this.auth_url;

            // Replace your application key below, which you have received from dataBridges management portal.
            this.dbridge.appkey = this.dbApplnKey;

            // Replace your appsecret below, which you have received from dataBridges management portal.
            this.dbridge.appsecret = this.dbApplnSecret;

            // Initialize RPC Server weather.now.
            this.weatherSvr = dbridge.rpc.init("weather.now");

            Action<object> ifunctions = this.rpcfunction;
            this.weatherSvr.functions = ifunctions;


            //Bind to connected event, Once the dataBridges is connected successfully, Subscription and other activities can be performed.
            Action<object> connected_callback;
            this.dbridge.connectionstate.bind("connected", connected_callback = async (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->connected" + "sessionid-> " + this.dbridge.sessionid);
                // Register RPC Server with dBrdige network.
                await this.weatherSvr.register();

                // Bind to server.registration.success, This will be triggered When summary is ready to serve.
                Action<object, object> rconnectsuccess;
                this.weatherSvr.bind("dbridges:rpc.server.registration.success", rconnectsuccess = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.success {0} {1} {2} {3}", payload, metadata, this.weatherSvr.getServerName(), this.weatherSvr.isOnline());
                });
                // Bind to server.online, This will be triggered When summary is ready to serve.
                Action<object, object> rconnectonline;
                this.weatherSvr.bind("dbridges:rpc.server.online", rconnectonline = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.online {0} {1} {2} {3}", payload, metadata, this.weatherSvr.getServerName(), this.weatherSvr.isOnline());
                });
                // Bind to server.registration.fail, This will be triggered When any registration issue is raised.
                Action<object, object> rconnectfail;
                this.weatherSvr.bind("dbridges:rpc.server.registration.fail", rconnectfail = (object payload, object metadata) => {
                    Console.WriteLine("dbridges:rpc.server.fail {0} {1} {2} {3}", payload, metadata, this.weatherSvr.getServerName(), this.weatherSvr.isOnline());
                });
            });

            //Bind to disconnected event, to get intimation about dataBridges network disconnection.
            Action<object> disconnected_callback;
            this.dbridge.connectionstate.bind("disconnected", disconnected_callback = (object eventinfo) => {
                Console.WriteLine("dbridge.connectionstate.bind: event->disconnected");
            });
            // Connect to dataBridges. If any runtime error it will be caught in catch().
            try
            {
                await this.dbridge.connect();
                string ch = Console.ReadLine();
            }
            catch (Exception e)
            {
                Console.WriteLine(e);
            }
        }
    }


    class Program
    {
        static void Main(string[] args)
        {
            CdBridgeServer db = new CdBridgeServer();
            Task.Run(db.init_dataBridges).Wait();
        }
    }
}